//!
//!

use super::models;
use crate::repositories;
use diesel::{result::Error, sql_query, RunQueryDsl};

static MEMBER_QUERY: &str = "
SELECT
  mr.id                     as mr_id,
  mr.member_id              as mr_member_id,
  mr.can_use                as mr_can_use,  
  mr.memo                   as mr_memo,    
  mr.created_at             as mr_created_at,
  mr.updated_at             as mr_updated_at,
  
  _m.id                     as _m_id,
  _m.site_id                as _m_site_id,
  _m.member_class_id        as _m_member_class_id,  
  _m.member_level_id        as _m_member_level_id,    
  _m.username               as _m_username,  
  _m.password               as _m_password,  
  _m.nickname               as _m_nickname,
  _m.mobile_phone_number    as _m_mobile_phone_number,
  _m.state                  as _m_state,
  _m.state_changed_at       as _m_state_changed_at,
  _m.parent_member_id       as _m_parent_member_id,
  _m.child_member_count     as _m_child_member_count,
  _m.last_signined_ip       as _m_last_signined_ip,
  _m.last_signined_at       as _m_last_signined_at,
  _m.created_at             as _m_created_at,
  _m.updated_at             as _m_updated_at,
  _m.deleted_at             as _m_deleted_at

FROM member_referrers as m
  INNER JOIN members _m
    ON _m.id = mr.member_id
";

pub struct Composition {}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Default for Composition {
  fn default() -> Self {
    Self::new()
  }
}

impl Composition {
  ///
  pub fn new() -> Composition {
    Composition {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::MemberReferrerModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      m.id = $1
    ",
      MEMBER_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(id)
      .get_result::<models::MemberReferrerModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &repositories::member_referrer::models::FindAll,
  ) -> Result<Vec<models::MemberReferrerModel>, Error> {
    use std::fmt::Write;

    let mut query = String::new();
    write!(&mut query, "{}", MEMBER_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let mut query_where = String::new();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mr.member_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = s.can_use {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mr.can_use = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.memo_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mr.memo like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }

    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_results::<models::MemberReferrerModel>(conn) {
      Ok(m) => Ok(m),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(vec![]),
        _ => Err(e),
      },
    }
  }
}
